#include "MorseCode.h"
/*
 * MorseCode.h - 
  
  Created 2017 by DonX-Developer
  This library is intended to be used solely with Arduino

  The library is free software; you can redistribute it and/or modify it as you see fit.

  
  This code was primarily created for an Arduino UNO using the Arduino IDE v1.6.9. 
  It has been tested on other Arduino boards but not extensively.
  
 */

MorseCode::MorseCode()	
{
	int led1; 
	int led2; 
	int morseKey; 
	int buzzer; 
	int unit; 
	long debounceDelay;

	bool showMorseCharacters; 
	bool showSentence; 
	bool showCharacters;
	bool showPhrase; 
	bool speakerOut; 
	bool ledOut;
	bool showCharacterGap;

}

const int DIT = 1;//unit length of a dit
const int DAH = 3;//unit length of a dah
const int CODEGAP = 1; //unit length of a gap between morse dits and dahs
const int LETTERGAP = 3; //unit length of a gap between letters
const int WORDGAP = 7; //unit length of a gap between words

int buttonState;//the current reading from the input pin
int lastButtonState = LOW;// the previous reading from the input pin
long lastDebounceTime = 0;// the last time the output pin was toggled
//reading the tapping of the key is how we determine the dit or dah
//however we also need to determine the time between dits and dahs for a character and word ending
//so we have to measure gaps of non-use, hence the start and end times
long startCode = 0; //start time for a possible key press
long endCode = 0; //end time for a possible key press
long startTime = 0; //start time for a non-key press
long endTime = 0; //end time for a non-key press
bool gotKeyPress = false;
int previousButtonState = 0;
bool endOfWord = false;
bool foundPhrase = false;

//variables to be available on the serial monitor
String characterStore = "";//hold the current text of the input morse code
String wordStore = ""; //hold the words as they are created
String sentenceStore = ""; //holds the current sentence as it is created
String currentCharacter;//holds the current character

//work out if a key was pressed and for how long
//a one unit would be a dit and a 3 unit would be a dah
void MorseCode::checkKeyPressType()
{
	long inactivity = 0; //test for gaps in code
	int milliUnit = unit * 1.1;//modify the unit to allow for code timing
	if (debounceDelay == 0)//quick and dirty debounce
	{
		debounceDelay = 80;
	}
	buttonState = digitalRead(morseKey);//see what, if anything, is happening 
	if (buttonState == LOW ) //key down
	{
		if (speakerOut)//beep if required
		{
			analogWrite(buzzer, 180);
		}
		if (ledOut)//glow if required (this is why we can only have one LED for the key press)
		{
			digitalWrite(led1, HIGH);
		}
		gotKeyPress = false;
		delay(debounceDelay);//allow for key debounce
	}
	else//key up
	{
		endCode = millis();//if key now up, catch the end time
	}
	if (buttonState == HIGH && startCode != endCode) //key up after being down
	{
		analogWrite(buzzer, 0);//turn off
		digitalWrite(led1, LOW);//turn off
		inactivity = endCode - startCode;//get the time of the length of key press

		if (inactivity > WORDGAP * milliUnit * 2  && !gotKeyPress) //long press so must be a reset
		{
			displayReset();
			resetOutput();
			gotKeyPress = true;
		}

		if (inactivity > milliUnit * 1.5   && !gotKeyPress) //must be a dah
		{
			characterStore += "-";//add the dah to the store
			if (showMorseCharacters)
			{
				Serial.println(" dah ");
			}
			resetTime();
			gotKeyPress = true;
		}
		if (inactivity >= milliUnit * 0.2 && inactivity <= milliUnit * 1.5 && !gotKeyPress) //must be a dit
		{
			characterStore += ".";//add the dit to the store
			if (showMorseCharacters)
			{
				Serial.println(" dit ");
			}
			resetTime();
			gotKeyPress = true;
		}
		resetCode();//reset the timing for the key press
		//**************************************************************************************
		//got the dit or dah, now work out the gap timings for letters and words
		if (characterStore.length() > 8 )//Oops too long (the longest is 7 + one for luck)
		{
			characterStore = "";//reset garbage input
		}
		if (gotKeyPress) //no point in checking this otherwise
		{
			endTime = millis(); //capture the time of inactivity
			inactivity = endTime - startTime; //difference between start and end is how long nothing has happened
			if (inactivity >= WORDGAP * milliUnit * 1 && !endOfWord) //end of word
			{
				foundPhrase = checkPhrases();//see if text matches a phrase
				if (!foundPhrase)
				{
					sentenceStore += wordStore + " "; //add word to sentence
					gotKeyPress = false;
				}
				wordStore = ""; //end of word
				characterStore = ""; //end of character
				endOfWord = true;
				doPrint(1, inactivity);//display current sentence text
			}
			if (inactivity >= LETTERGAP * milliUnit * 1.0 && characterStore != "") //end of character
			{
				currentCharacter = findCharacter();//get the current character
				if (currentCharacter != "")
				{
					wordStore += currentCharacter; //add the current character to the wordstore
					resetTime(); //reset
					doPrint(2, inactivity);//display current sentence character
					characterStore = ""; //reset
					endOfWord = false;
				}
			}
		}
	}
}

//reset variables
void MorseCode::resetOutput()
{
	resetTime();
	resetCode();
	characterStore = "";
	wordStore = "";
	sentenceStore = "";
	gotKeyPress = false;
	doPrint(0, 0);		
}

//do some output
void MorseCode::doPrint(int type, int inactivity)
{
	switch (type)
	{
	case 0:
		{
			Serial.print("Ready..... ");
			break;
		}
	case 1:
		{
			if(showSentence)
			{
				Serial.println();
				Serial.print(" > Sentence: ");
				Serial.println(sentenceStore);
			}
			break;
		}
	case 2:
		{
			if (showCharacters)
			{
				Serial.print(" > Char: ");
				Serial.print(currentCharacter);
			}
			if(showCharacterGap)
			{
				Serial.print(" > Character gap ");
			}
			break;
		}
	}
	Serial.println();
}

//convert a boolean input to a On/Off text output
String MorseCode::convertBool(bool value)
{
	return value ? "On" : "Off";
}

//reset message
void MorseCode::displayReset()
{
	Serial.println();
	Serial.println();
	Serial.println("Resetting");
	Serial.println("Morse Chars: " + convertBool(showMorseCharacters));
	Serial.println("Sentence Text: " + convertBool(showSentence));
	Serial.println("Chars: " + convertBool(showCharacters));
	Serial.println("Phrase: " + convertBool(showPhrase));
	Serial.println("Speaker: " + convertBool(speakerOut));
	Serial.println("LED(s): " + convertBool(ledOut));
	Serial.println();
	Serial.println();
}

//reset the time for the gaps
void MorseCode::resetTime()
{
	startTime = millis();
	endTime = startTime;
}

//reset the time for the code lengths
void MorseCode::resetCode()
{
	startCode = millis();
	endCode = startCode;
}

//the list of characters that we have codes for
static const char *letters =
	"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,?'!/()&:;=+-_\"$@ ";

//the codes that we have characters for, in the same order as the letters above
//0 = dah, 1 = dit
static const char *codes[] = {
	".-",     //A
	"-...",   //B
	"-.-.",   //C
	"-..",    //D
	".",      //E
	"..-.",   //F
	"--.",    //G
	"....",   //H
	"..",     //I
	".---",   //J
	"-.-",    //K
	".-..",   //L
	"--",     //M
	"-.",     //N
	"---",    //O
	".--.",   //P
	"--.-",   //Q
	".-.",    //R
	"...",    //S
	"-",      //T
	"..-",    //U
	"...-",   //V
	".--",    //W
	"-..-",   //X
	"-.--",   //Y
	"--..",   //Z
	"-----",  //0
	".----",  //1
	"..---",  //2
	"...--",  //3
	"....-",  //4
	".....",  //5
	"-....",  //6
	"--...",  //7
	"---..",  //8
	"----.",  //9
	".-.-.-", //.
	"--..--", //,
	"..--..", //?
	".----.", //'
	"-.-.--", //!
	"-..-.",  ///
	"-.--.",  //(
	"-.--.-", //)
	".-...",  //&
	"---...", //:
	"-.-.-.", //;
	"-...-",  //=
	".-.-.",  //+
	"-...-",  //-
	"..--.-", //_
	".-..-.", //"
	"...-..-",//$
	".--.-.", //@
	""        //space
};

//the list of 34 common phrases in use
static const char *phrases[] = {
	"I acknowledge",							 //QSL
	"Do you acknowledge?",                       //QSL?
	"Wait",                                      //QRX
	"Should I wait?",                            //QRX?
	"I am ready to copy",                        //QRV
	"Are you ready to copy?",                    //QRV?
	"The frequency is in use",                   //QRL
	"Is the frequency in use?",                  //QRL?
	"My location is",                            //QTH
	"What is your location?",                    //QTH?
	"New line",                                  //AA
	"New page",                                  //AR
	"Wait",                                      //AS
	"Break",                                     //BK
	"New paragraph",                             //BT
	"Going off the air",						 //CL
	"Attention",                                 //CT
	"Change to Wabun code",                      //DO
	"Invite station to transmit",				 //KN
	"End of transmission",                       //SK
	"Understood",                                //SN
	"Understood",                                //VE
	"Distress",			                         //SOS
	"Over",                                      //K
	"Roger",                                     //R
	"See you later",                             //CUL
	"Be seeing you",                             //BCNU
	"You're",                                    //UR
	"Signal report",                             //RST
	"Best regards",                              //73
	"Love and kisses",                           //88
	"Error",                                     //HH
	"Error",                                     //????
	"Change to Wabun code"                       //NJ
};

//the list of 34 phrase codes for above
static const char *phraseCodes[] = {
	"QSL",
	"QSL?",
	"QRX",
	"QRX?",
	"QRV",
	"QRV?",
	"QRL",
	"QRL?",
	"QTH",
	"QTH?",
	"AA",
	"AR",
	"AS",
	"BK",
	"BT",
	"CL",
	"CT",
	"DO",
	"KN",
	"SK",
	"SN",
	"VE",
	"SOS",
	"K",
	"R",
	"CUL",
	"BCNU",
	"UR",
	"RST",
	"73",
	"88",
	"HH",
	"????",
	"NJ"
};

//check if the current word matches a phrase
bool MorseCode::checkPhrases()
{
	bool phraseFound = false;
	if (showPhrase)//is it turned on
	{
		for (unsigned  int code = 0; code < 34; ++code )
		{
			if (String(phraseCodes[code]) == wordStore ) //found phrase code
			{
				sentenceStore += String(phrases[code]) + " ";//add the phrase to the sentence
				phraseFound = true;
				break;//no need to carry on so return to calling function
			}
		}
	}
	return phraseFound;
}

//turn the LED on/off for Serial incoming code
void MorseCode::sendToLed(int code)
{
	Serial.println();//new character
	Serial.print(letters[code]);
	Serial.print("   ");
	for (unsigned  int i = 0; i < strlen(codes[code]); ++i )
	{
		if ( i > 0 )//put a gap between dits and dahs
		{
			morseGap(CODEGAP);
		}
		if ( codes[code][i] == '.' )//display a dit
		{
			morseBeep(DIT);
			Serial.print(".");
		}
		if ( codes[code][i] == '-' )//display a dah
		{
			morseBeep(DAH);
			Serial.print("-");
		}
	}
	morseGap(LETTERGAP);//end of character
	Serial.println();
}

//loop through the codes to find a match
bool MorseCode::getMorse(char character)
{
	bool characterFound = false;
	if(character != char())
	{
		character=toUpperCase(character);
		for (unsigned  int code = 0; code < strlen(letters); ++code )
		{
			if (letters[code] == character ) //found character
			{
				sendToLed(code);//got the code for the character
				characterFound = true;
				break;//no need to carry on so return to calling function
			}
		}
	}
	return characterFound;
}

//based on the dits and dahs held in characterStore, find the corresponding letter
String MorseCode::findCharacter()
{
	for (unsigned  int i = 0; i < 55; ++i )
	{
		String code = String(codes[i]);
		String letter = String(letters[i]);
		if (code == characterStore) //current character found
		{
			return letter;//no need to look any more
		}
	}
	return "";
}

//turn the LEDs off
void MorseCode::ledOff()
{
	digitalWrite(led1, LOW);
	digitalWrite(led2, LOW);
}

//define the gaps between dits, dahs, letters and words
//for the LED
void MorseCode::morseGap(int gapLength)
{
	ledOff();
	delay(unit * gapLength);
}
//show the incoming morse dit or dah via the LED and/or the speaker
void MorseCode::morseBeep(int unitLength)
{
	int currentLed=0;
	if(unitLength==DIT)//turn on the dit LED
	{
		currentLed=led1;
	}
	else//turn on the dah LED
	{
		currentLed=led2;
	}
	if (ledOut)
	{
		digitalWrite(currentLed, HIGH);
	}
	if (speakerOut)
	{
		analogWrite(buzzer, 180);
	}
	delay(unitLength * unit);
	analogWrite(buzzer, 0);
	ledOff();
}
