/*
 * MorseCode.h - 
  
  Created 2017 by DonX-Developer
  This library is intended to be used solely with Arduino

  The library is free software; you can redistribute it and/or modify it as you see fit.

  
  This code was primarily created for an Arduino UNO using the Arduino IDE v1.6.9. 
  It has been tested on other Arduino boards but not extensively.
  
 */

#ifndef MorseCode_h
#define MorseCode_h

#if ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#include "pins_arduino.h"
#include "WConstants.h"
#endif

class MorseCode{
public:
	MorseCode();

	int led1;//Arduino pin that the LED is connected to
	int led2;//Arduino pin that the LED is connected to
	int morseKey;//morse code key (or momentary switch)
	int buzzer;//Arduino pin that the buzzer is connected to		
	int unit;//the length of a unit, defines the length of dits dahs and spaces etc
	long debounceDelay;//the debounce time
	bool showMorseCharacters;//display dit or dah after entry
	bool showSentence;//display the current sentence so far
	bool showCharacters;//display the current character between letters
	bool showPhrase;//display certain built in phrases
	bool speakerOut;//turn speaker on/off
	bool ledOut;//turn leds on/off
	bool showCharacterGap;//helps with the start of the next letter

	void checkKeyPressType();
	void resetOutput();
	bool getMorse(char character);

private:

	void doPrint(int type, int inactivity);
	String convertBool(bool value);
	void displayReset();
	void resetTime();
	void resetCode();
	bool checkPhrases();
	void sendToLed(int code);
	
	String findCharacter();
	void ledOn(int unitLength);
	void ledOff();
	void morse(int unitLength);
	void morseGap(int gapLength);
	void morseBeep(int unitLength);
};

#endif